#include <stdint.h>
#include <stdbool.h>

#include "log.h"
#include "net.h"
#include "action.h"
#include "udp.h"
#include "ntpclient.h"
#include "ntpserver.h"
#include "clktime.h"
#include "ntp.h"
#include "fault.h"


bool NtpTrace = false;

void NtpInit()
{
   NtpClientInit();
}

void NtpLogHeader(struct NtpHeader* pHeader)
{
    if (NetTraceVerbose) Log ("NTP header\r\n  ");
    else                 Log ("NTP   header: ");
    
    LogF("Mode %d",         pHeader->Mode);
    LogF(", Version %d",    pHeader->VN);
    LogF(", LI %d",         pHeader->LI);
    LogF(", Stratum %d",    pHeader->Stratum);
    LogF(", Poll %d",       pHeader->Poll);
    LogF(", Precision %d",  pHeader->Precision);
    LogF(", Root delay %d", NetToHost32(pHeader->RootDelay));
    LogF(", Dispersion %d", NetToHost32(pHeader->Dispersion));
    Log (", Ident ");
    for (int i = 0; i < 4; i++) if (pHeader->RefIdentifier[i]) LogChar(pHeader->RefIdentifier[i]);
    Log ("\r\n");
    
    if (NetTraceVerbose)
    {
        LogF("  REF %llu\r\n",  NetToHost64(pHeader->RefTimeStamp));
        LogF("  ORI %llu\r\n",  NetToHost64(pHeader->OriTimeStamp));
        LogF("  REC %llu\r\n",  NetToHost64(pHeader->RecTimeStamp));
        LogF("  TRA %llu\r\n",  NetToHost64(pHeader->TraTimeStamp));
    }
}

int NtpHandlePacketReceived(void (*traceback)(void), int sizeRx, void * pPacketRx, int* pSizeTx, void* pPacketTx)
{
    int lastFaultPoint = FaultPoint;
    FaultPoint = FAULT_POINT_NtpHandlePacketReceived;
    
    if (sizeRx != sizeof(struct NtpHeader))
    {
        LogTimeF("\r\nNTP packet wrong size %d\r\n", sizeRx);
        return DO_NOTHING;
    }
    struct NtpHeader* pHeaderRx = (struct NtpHeader*)pPacketRx;
    struct NtpHeader* pHeaderTx = (struct NtpHeader*)pPacketTx;
    
    int dest = DO_NOTHING;
    switch (pHeaderRx->Mode)
    {
        case NTP_CLIENT: dest = NtpServerRequest(traceback, pHeaderRx, pHeaderTx);        break;
        case NTP_SERVER:        NtpClientReply  (traceback, pHeaderRx);                   break;
        default:         LogTimeF("\r\nNTP packet unknown mode %d\r\n", pHeaderRx->Mode); break;
    }
    
    FaultPoint = lastFaultPoint;
    if (dest == DO_NOTHING)
    {
        return DO_NOTHING;
    }
    else
    {
        *pSizeTx = sizeof(struct NtpHeader);
        return ActionMakeFromDestAndTrace(dest, NtpTrace && NetTraceStack);
    }
}
int NtpPollForPacketToSend(int type, void* pPacket, int* pSize)
{    
    int dest = NtpClientQueryPoll(type, pPacket, pSize);
    if (!dest) return DO_NOTHING;
    
    if (NtpTrace)
    {
        if (NetTraceNewLine) Log("\r\n");
        LogTimeF("Sending NTP request\r\n");
        NtpLogHeader((struct NtpHeader*)pPacket);
    }
    return ActionMakeFromDestAndTrace(dest, NtpTrace && NetTraceStack);


}

